package com.cc.tools.utils

import android.content.Context
import android.os.Environment
import android.os.StatFs
import java.text.DecimalFormat


/**
 * The helper class used to check the available space.
 */
object Storage {

    /**
     * Get the available space of the Android Download/Cache content directory.
     *
     * @return available space
     */
    fun cachePartitionAvailableSpace(): Long {
        val stat = StatFs(Environment.getDownloadCacheDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val availableBlocks = stat.availableBlocks.toLong()
        return availableBlocks * blockSize
    }

    /**
     * Get the available space of the Android data directory.
     *
     * @return available space
     */
    fun dataPartitionAvailableSpace(): Long {
        val stat = StatFs(Environment.getDataDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val availableBlocks = stat.availableBlocks.toLong()
        return availableBlocks * blockSize
    }

    fun recordAvailableSpace(context: Context): Long {
        if (externalStorageAvailable()) {
            val filePath = Environment.getExternalStorageDirectory().getAbsolutePath()
            val stat = StatFs(filePath)
            val blockSize = stat.blockSize.toLong()
            val availableBlocks = stat.availableBlocks.toLong()
            return availableBlocks * blockSize
        } else {
            return dataPartitionAvailableSpace()
        }
    }

    /**
     * To check whether the Android external storage directory is available or not.
     *
     * @return
     */
    fun externalStorageAvailable(): Boolean {
        val state = Environment.getExternalStorageState()
        return Environment.MEDIA_MOUNTED.equals(state)
    }

    /**
     * 获取手机内部可用空间大小
     *
     * @return
     */
    fun internalStorageAvailableSpace(): Long {
        val stat = StatFs(Environment.getDataDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val availableBlocks = stat.availableBlocks.toLong()
        return availableBlocks * blockSize
    }

    /**
     * 获取手机内部空间大小
     *
     * @return
     */
    fun internalStorageTotalSpace(): Long {
        val stat = StatFs(Environment.getDataDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val blockCount = stat.blockCount.toLong()
        return blockCount * blockSize
    }

    /**
     * 获取手机外部可用空间大小
     *
     * @return
     */
    fun externalStorageAvailableSpace(): Long {
        if (!externalStorageAvailable()) {
            return -1
        }
        val stat = StatFs(Environment.getExternalStorageDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val availableBlocks = stat.availableBlocks.toLong()
        return availableBlocks * blockSize
    }

    /**
     * 获取手机外部空间大小
     *
     * @return
     */
    fun externalStorageTotalSpace(): Long {
        if (!externalStorageAvailable()) {
            return -1
        }
        val stat = StatFs(Environment.getExternalStorageDirectory().getPath())
        val blockSize = stat.blockSize.toLong()
        val blockCount = stat.blockCount.toLong()
        return blockCount * blockSize
    }

    /**
     * @param size The size to format.
     * @return
     */
    fun readableSize(size: Long): String {
        if (size <= 0) {
            return "0"
        }
        val units = arrayOf("B", "KB", "MB", "GB", "TB")
        val digitGroups = (Math.log10(size.toDouble()) / Math.log10(1024.0)).toInt()
        return DecimalFormat("#,##0.00").format(size / Math.pow(1024.0, digitGroups.toDouble())) + units[digitGroups]
    }
}
